// Superpro.cpp : Implementation of CSuperpro
/***************************************************************************************
*SuperPro.cpp
*--------------------------------------------------------------------------------------
*
* Description - Implementation file for CSuperPro Class
*
* Created By  - Jogender Chauhan.
*
* Purpose     - Definition for all the member functions of this class
*				
* (C) Copyright 1991-2002 Rainbow Technologies, Inc. All rights reserved.
*-------------------------------------------------------------------------------------------
*
******************************************************************************************/

#include "stdafx.h"
#include "RainbowSSP.h"
#include "Superpro.h"

#define MAX_SERVERS 32

const IID NSPRO_MONITOR_INFO_T_IID = { 0xBBD73A31, 0x8504, 0x11d5, { 0x9C, 0x72, 0x00, 0x90, 0x27, 0xCB, 0x09, 0xF5 }};
const IID NSPRO_SERVER_INFO_T_IID = {	0xF05CE561, 0x8089, 0x11d5, { 0x9C, 0x70, 0x00, 0x90, 0x27, 0xCB, 0x09, 0xF5 }};   


/////////////////////////////////////////////////////////////////////////////
// CSuperpro

STDMETHODIMP CSuperpro::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ISuperpro,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CSuperpro::OnStartPage (IUnknown* pUnk)  
{
	if(!pUnk)
		return E_POINTER;

	CComPtr<IScriptingContext> spContext;
	HRESULT hr;

	// Get the IScriptingContext Interface
	hr = pUnk->QueryInterface(IID_IScriptingContext, (void **)&spContext);
	if(FAILED(hr))
		return hr;

	// Get Request Object Pointer
	hr = spContext->get_Request(&m_piRequest);
	if(FAILED(hr))
	{
		spContext.Release();
		return hr;
	}

	// Get Response Object Pointer
	hr = spContext->get_Response(&m_piResponse);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		return hr;
	}
	
	// Get Server Object Pointer
	hr = spContext->get_Server(&m_piServer);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		return hr;
	}
	
	// Get Session Object Pointer
	hr = spContext->get_Session(&m_piSession);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		return hr;
	}

	// Get Application Object Pointer
	hr = spContext->get_Application(&m_piApplication);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		m_piSession.Release();
		return hr;
	}
	m_bOnStartPageCalled = TRUE;
	return S_OK;
}

STDMETHODIMP CSuperpro::OnEndPage ()  
{
	m_bOnStartPageCalled = FALSE;
	// Release all interfaces
	m_piRequest.Release();
	m_piResponse.Release();
	m_piServer.Release();
	m_piSession.Release();
	m_piApplication.Release();

	return S_OK;
}


STDMETHODIMP CSuperpro::FindFirstUnit(VARIANT developerId)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (developerId.vt == VT_EMPTY)
		return E_INVALIDARG;
	//FinalConstruct();    //Jogender
	m_LastError = m_SuperproContainer->FindFirstUnit(developerId);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: FormatPacket()
*
*	PURPOSE		: Format the packet for a client
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the S_OK if no error or E_UNEXPECTED
*
*********************************************************************************/


STDMETHODIMP CSuperpro::FormatPacket()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (m_SuperproContainer == NULL)
		m_SuperproContainer = new CSuperproContainer();

	if (m_SuperproContainer != NULL)
	{
		m_LastError = m_SuperproContainer->FormatPacket();
		if (m_LastError != SP_SUCCESS)
		{
			delete m_SuperproContainer;
			m_SuperproContainer = NULL;
			return E_UNEXPECTED;
		}
	} else 
		return E_UNEXPECTED;

	return S_OK;
}

/********************************************************************************
*
*	FUNCTION	: Initialize()
*
*	PURPOSE		: Initialize the SuperPro packet for a client
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the S_OK if no error or E_UNEXPECTED
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Initialize()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (m_SuperproContainer == NULL)
	{
		m_SuperproContainer = new CSuperproContainer();
		if (m_SuperproContainer != NULL)
		{
			m_LastError = m_SuperproContainer->FormatPacket();
			if (m_LastError != SP_SUCCESS)
			{
				delete m_SuperproContainer;
				m_SuperproContainer = NULL;
				return E_UNEXPECTED;
			}
		}
	}

	m_LastError = m_SuperproContainer->Initialize();

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: get_LastError()
*
*	PURPOSE		: To get Last Error return by client Library
*
*	INPUT/OUTPUT: [IN/OUT] pVal
*
*	RETURN		: Return the S_OK if no error or E_POINTER
*
*********************************************************************************/


STDMETHODIMP CSuperpro::get_LastError(short *pVal)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (!pVal)
		return E_POINTER;

	*pVal = m_LastError;

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetVersion()
*
*	PURPOSE		: Get the version of driver
*
*	INPUT/OUTPUT: returns Major Version,Minor version, Revision and OS type
*
*	RETURN		: Return the S_OK if no error or E_POINTER
*
*
*********************************************************************************/

STDMETHODIMP CSuperpro::GetVersion(VARIANT *pMajor, VARIANT *pMinor, VARIANT *pRev, VARIANT *pOsType)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (!pMajor)
		return E_POINTER;
	if (!pMinor)
		return E_POINTER;
	if (!pRev)
		return E_POINTER;
	if (!pOsType)
		return E_POINTER;

	HRESULT hr = ::VariantClear(pMajor);
	if (FAILED(hr))
		return hr;
	hr = ::VariantClear(pMinor);
	if (FAILED(hr))
		return hr;
	hr = ::VariantClear(pRev);
	if (FAILED(hr))
		return hr;
	hr = ::VariantClear(pOsType);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->GetVersion(pMajor, pMinor, pRev, pOsType);
	return S_OK;
}

/********************************************************************************
*
*	FUNCTION	: SetContactServer()
*
*	PURPOSE		: Set the name of server.
*
*	INPUT/OUTPUT: [IN] server name
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::SetContactServer(VARIANT serverName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (serverName.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->SetContactServer(serverName);
	return S_OK;

}
/********************************************************************************
*
*	FUNCTION	: Activate()
*
*	PURPOSE		: To Activate a cell.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] address of cell
*				  [IN] activate Password1 of key, [IN] activate Password2
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Activate(VARIANT writePassword, VARIANT activatepw1, VARIANT activatepw2, VARIANT address)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (writePassword.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (activatepw1.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (activatepw2.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->Activate(writePassword, activatepw1, activatepw2, address);

	return S_OK;
}

/********************************************************************************
*
*	FUNCTION	: Decrement()
*
*	PURPOSE		: To decrement the value of a cell.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] address of cell
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Decrement(VARIANT writePassword, VARIANT address)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (writePassword.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->Decrement(writePassword, address);


	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: FindFirstUnit()
*
*	PURPOSE		: For finding the key for a client
*
*	INPUT/OUTPUT: [IN] Dev ID of the key
*
*	RETURN		: Return the S_OK if no error
*
*********************************************************************************/

STDMETHODIMP CSuperpro::FindNextUnit()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_LastError = m_SuperproContainer->FindNextUnit();

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetContactServer()
*
*	PURPOSE		: Get the name of server.
*
*	INPUT/OUTPUT: [OUT] server name
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::GetContactServer(VARIANT *pServerName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (!pServerName)
		return E_POINTER;
	if (pServerName->vt != VT_EMPTY)
		return E_INVALIDARG;

	HRESULT hr = ::VariantClear(pServerName);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->GetContactServer(pServerName);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetFullStatus()
*
*	PURPOSE		: Get Last Error of API
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the S_OK
*
*********************************************************************************/

STDMETHODIMP CSuperpro::GetFullStatus()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	m_LastError = m_SuperproContainer->GetFullStatus();
	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetHardLimit()
*
*	PURPOSE		: Get the Hard Limit of a key
*
*	INPUT/OUTPUT: [OUT] HardLimit
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::GetHardLimit(VARIANT *pHardLimit)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (!pHardLimit)
		return E_POINTER;

	if (pHardLimit->vt != VT_EMPTY)
		return E_INVALIDARG;

	HRESULT hr = ::VariantClear(pHardLimit);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->GetHardLimit(pHardLimit);
	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetSubLicense()
*
*	PURPOSE		: Get a sub license for a cell
*
*	INPUT/OUTPUT: [IN] Cell address
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::GetSubLicense(VARIANT address)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->GetSubLicense(address);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: Overwrite()
*
*	PURPOSE		: To Overwrite a cell by a given value.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] overwrite password 1
*				  [IN] overwrite password 2, [IN] Address of cell
*				  [IN] data, [IN] Access Code of the cell
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Overwrite(VARIANT writePassword, VARIANT owp1, VARIANT owp2, VARIANT address, VARIANT data, VARIANT accessCode)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	if (writePassword.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (data.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (accessCode.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (owp1.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (owp2.vt == VT_EMPTY)
		return E_INVALIDARG;
	
	m_LastError = m_SuperproContainer->Overwrite(writePassword, owp1, owp2, address, data, accessCode);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: Read()
*
*	PURPOSE		: To Read a value from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [OUT] Data
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Read(VARIANT address, VARIANT *pData)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (!pData)
		return E_POINTER;

	HRESULT hr = ::VariantClear(pData);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->Read(address, pData);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: ReleaseLicense()
*
*	PURPOSE		: To ReleaseLicense a license from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [IN/OUT] license to release/Actual license released
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::ReleaseLicense(VARIANT address, VARIANT *pNumOfLic)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (!pNumOfLic)
		return E_POINTER;
	if (pNumOfLic->vt == VT_EMPTY)
	{
		::VariantClear(pNumOfLic);
		pNumOfLic->vt = VT_I2;
		pNumOfLic->iVal = 0;
	}
	
	m_LastError = m_SuperproContainer->ReleaseLicense(address, pNumOfLic);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: SetHeartBeat()
*
*	PURPOSE		: Set Heart beat for client..
*
*	INPUT/OUTPUT: [IN] heart beat value.
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::SetHeartBeat(VARIANT heartBeatValue)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	
	if (heartBeatValue.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->SetHeartBeat(heartBeatValue);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: Write()
*
*	PURPOSE		: To write a cell by a given value.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] Address of cell
*				  [IN] data, [IN] Access Code of the cell
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Write(VARIANT writePassword, VARIANT address, VARIANT data, VARIANT accessCode)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	if (writePassword.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (data.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (accessCode.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->Write(writePassword, address, data, accessCode);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: ExtendedRead()
*
*	PURPOSE		: To Read a value from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [OUT] Data, [OUT] Access code
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::ExtendedRead(VARIANT address, VARIANT *pData, VARIANT *pAccessCode)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (!pData || !pAccessCode)
		return E_POINTER;

	HRESULT hr = ::VariantClear(pData);
	if (FAILED(hr))
		return hr;
	hr = ::VariantClear(pAccessCode);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->ExtendedRead(address, pData, pAccessCode);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: SetProtocol()
*
*	PURPOSE		: Set the protocol for client/server communication.
*
*	INPUT/OUTPUT: [IN] Protocol
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::SetProtocol(VARIANT protocol)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (protocol.vt == VT_EMPTY)
		return E_INVALIDARG;

	m_LastError = m_SuperproContainer->SetProtocol(protocol);

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: Query()
*
*	PURPOSE		: For query on a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [IN] query Data, [OUT] Response
*				  [OUT] response32 (last 4 byte of response), [OUT] length of response
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::Query(VARIANT address, VARIANT queryData, VARIANT *pResponse, VARIANT *pResponse32, VARIANT length)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (address.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (queryData.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (length.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (!pResponse)
		return E_POINTER;
	if (!pResponse32)
		return E_POINTER;

	HRESULT hr = ::VariantClear(pResponse);
	if (FAILED(hr))
		return hr;
	hr = ::VariantClear(pResponse32);
	if (FAILED(hr))
		return hr;

	m_LastError = m_SuperproContainer->Query(address, queryData, pResponse, pResponse32,length);
	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: GetKeyInfo()
*
*	PURPOSE		: For getting the information of key.
*
*	INPUT/OUTPUT: [IN] Dev ID, [IN] Key Index, [OUT] structure of key info
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/


STDMETHODIMP CSuperpro::GetKeyInfo(VARIANT developerId, VARIANT keyIndex, NSPRO_MONITOR_INFO_T *pMonitorInfo)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	if (developerId.vt == VT_EMPTY)
		return E_INVALIDARG;
	if (keyIndex.vt == VT_EMPTY)
		return E_INVALIDARG;

	if (!pMonitorInfo)
         return E_POINTER;

	NSPRO_MONITOR_INFO bufferInfo;

 	m_LastError = m_SuperproContainer->GetKeyInfo(developerId, keyIndex, &bufferInfo);

	if (!m_LastError)
	{

		wchar_t *pValue = NULL;
		pMonitorInfo->serverName.vt = VT_BSTR;
		pValue = CSuperproContainer::MBCSToWideChar(bufferInfo.serverName);
		pMonitorInfo->serverName.bstrVal = SysAllocString(pValue);
		free(pValue); pValue = NULL;

		pMonitorInfo->serverIPAddress.vt = VT_BSTR;
		pValue = CSuperproContainer::MBCSToWideChar(bufferInfo.serverIPAddress);
		pMonitorInfo->serverIPAddress.bstrVal = SysAllocString(pValue);
		free(pValue); pValue = NULL;

		pMonitorInfo->serverIPXAddress.vt = VT_BSTR;
		pValue = CSuperproContainer::MBCSToWideChar(bufferInfo.serverIPXAddress);
		pMonitorInfo->serverIPXAddress.bstrVal = SysAllocString(pValue);
		free(pValue); pValue = NULL;

		pMonitorInfo->serverVersion.vt = VT_BSTR;
		pValue = CSuperproContainer::MBCSToWideChar(bufferInfo.version);
		pMonitorInfo->serverVersion.bstrVal = SysAllocString(pValue);
		free(pValue); pValue = NULL;

		pMonitorInfo->serverVersion.vt = VT_I2;
		pMonitorInfo->protocol.iVal = bufferInfo.protocol;

		pMonitorInfo->keyInfo.developerId.vt = VT_I2;
		pMonitorInfo->keyInfo.developerId.lVal = bufferInfo.sproKeyMonitorInfo.devId;

		pMonitorInfo->keyInfo.hardLimit.vt = VT_I2;
		pMonitorInfo->keyInfo.hardLimit.iVal = bufferInfo.sproKeyMonitorInfo.hardLimit;

		pMonitorInfo->keyInfo.inUse.vt = VT_I2;
		pMonitorInfo->keyInfo.inUse.iVal = bufferInfo.sproKeyMonitorInfo.inUse;

		pMonitorInfo->keyInfo.numTimeOut.vt = VT_I2;
		pMonitorInfo->keyInfo.numTimeOut.iVal = bufferInfo.sproKeyMonitorInfo.numTimeOut;

		pMonitorInfo->keyInfo.highestUse.vt = VT_I2;
		pMonitorInfo->keyInfo.highestUse.iVal = bufferInfo.sproKeyMonitorInfo.highestUse;
	}

	return S_OK;
}
/********************************************************************************
*
*	FUNCTION	: EnumServer()
*
*	PURPOSE		: For getting the list of running servers in subnet.
*
*	INPUT/OUTPUT: [IN] flag , [IN] Dev ID, [OUT] structure of server list
*
*	RETURN		: Return the S_OK if no error or E_INVALIDARG
*
*********************************************************************************/

STDMETHODIMP CSuperpro::EnumServer(VARIANT eFlag, VARIANT devId, NSPRO_SERVER_INFO_T *pServerInfo, VARIANT *pNumOfServers)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	
	//VARIANT NumOfServers;

	if (eFlag.vt == VT_EMPTY)
		return E_INVALIDARG;

	if (devId.vt == VT_EMPTY)
		return E_INVALIDARG;

	if ( (!pServerInfo) || (!pNumOfServers) )
         return E_POINTER;
 
	NSPRO_SERVER_INFO bufferInfo[32];
	
	m_LastError = m_SuperproContainer->EnumServer(eFlag, devId, bufferInfo, pNumOfServers);

 	RB_WORD dwNumServer = 0;
	
	if (!m_LastError)
	{
		wchar_t *pTemp = NULL;
		
		dwNumServer = pNumOfServers->intVal;

		for (unsigned int i=0; i < dwNumServer; i++)
		{
			pServerInfo[i].serverAddress.vt = VT_BSTR;
			pTemp = CSuperproContainer::MBCSToWideChar(bufferInfo[i].serverAddress);
			pServerInfo[i].serverAddress.bstrVal = SysAllocString(pTemp);
			free(pTemp); pTemp = NULL;

			pServerInfo[i].numOfLicense.vt = VT_INT;
			pServerInfo[i].numOfLicense.intVal = bufferInfo[i].numLicAvail;
			free(pTemp); pTemp = NULL;
		}
	}
	return S_OK;
}
