	
// Superpro.h : Declaration of the CSuperpro
/***************************************************************************************
*SuperPro.h
*--------------------------------------------------------------------------------------
*
* Description - Header file for CSuperPro Class
*
* Created By  - Jogender Chauhan.
*
* Purpose     - This Class is for all the API supported by SuperPro.
*				
* (C) Copyright 1991-2002 Rainbow Technologies, Inc. All rights reserved.
*-------------------------------------------------------------------------------------------
*
******************************************************************************************/

#ifndef __SUPERPRO_H_
#define __SUPERPRO_H_

#include "resource.h"       // main symbols
#include <asptlb.h>         // Active Server Pages Definitions
#include "SuperproContainer.h"	// Added by ClassView
#include "RainbowSSPCP.h"

/////////////////////////////////////////////////////////////////////////////
// CSuperpro
class ATL_NO_VTABLE CSuperpro : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CSuperpro, &CLSID_Superpro>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CSuperpro>,
	public IDispatchImpl<ISuperpro, &IID_ISuperpro, &LIBID_RAINBOWSSPLib>,
	public CProxy_ISuperproEvents< CSuperpro >
{
public:
	CSuperpro()
	{ 
		m_SuperproContainer = NULL;
		m_pUnkMarshaler = NULL;
		m_bOnStartPageCalled = FALSE;
	}

public:

DECLARE_REGISTRY_RESOURCEID(IDR_SUPERPRO)

DECLARE_PROTECT_FINAL_CONSTRUCT()
DECLARE_GET_CONTROLLING_UNKNOWN()

BEGIN_COM_MAP(CSuperpro)
	COM_INTERFACE_ENTRY(ISuperpro)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CSuperpro)
CONNECTION_POINT_ENTRY(DIID__ISuperproEvents)
END_CONNECTION_POINT_MAP()


	HRESULT FinalConstruct()
	{
		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}

	void FinalRelease()
	{
		if(m_SuperproContainer != NULL)
		{
			delete m_SuperproContainer;
			m_SuperproContainer = NULL;
		}
		m_pUnkMarshaler.Release();
	}

	CComPtr<IUnknown> m_pUnkMarshaler;

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// ISuperpro
public:
	STDMETHOD(EnumServer)(/*[in]*/ VARIANT eFlag, /*[in]*/ VARIANT devId, /*[in,out]*/ NSPRO_SERVER_INFO_T *pServerInfo, /*[in,out]*/ VARIANT *pNumOfServers);
	STDMETHOD(GetKeyInfo)(/*[in]*/ VARIANT developerId, /*[in]*/ VARIANT keyIndex, /*[in,out]*/ NSPRO_MONITOR_INFO_T *pMonitorInfo);
	STDMETHOD(Query)(/*[in]*/ VARIANT address, /*[in]*/ VARIANT queryData, /*[in,out]*/ VARIANT *pResponse, /*[in,out]*/ VARIANT *pResponse32, /*[in]*/ VARIANT length);
	STDMETHOD(SetProtocol)(/*[in]*/ VARIANT protocol);
	STDMETHOD(ExtendedRead)(/*[in]*/ VARIANT address, /*[in,out]*/ VARIANT *pData, /*[in,out]*/ VARIANT *pAccessCode);
	STDMETHOD(Write)(/*[in]*/ VARIANT writePassword, /*[in]*/ VARIANT address, /*[in]*/ VARIANT data, /*[in]*/ VARIANT accessCode);
	STDMETHOD(SetHeartBeat)(/*[in]*/VARIANT heartBeatValue);
	STDMETHOD(ReleaseLicense)(/*[in]*/ VARIANT address, /*[in,out]*/ VARIANT *pNumOfLic);
	STDMETHOD(Read)(/*[in]*/ VARIANT address, /*[in,out]*/ VARIANT *pData);
	STDMETHOD(Overwrite)(/*[in]*/ VARIANT writePassword, /*[in]*/ VARIANT owp1, /*[in]*/ VARIANT owp2, /*[in]*/ VARIANT address, /*[in]*/ VARIANT data, /*[in]*/ VARIANT accessCode);
	STDMETHOD(GetSubLicense)(/*[in]*/ VARIANT address);
	STDMETHOD(GetHardLimit)(/*[out,retval]*/ VARIANT *pHardLimit);
	STDMETHOD(GetFullStatus)();
	STDMETHOD(GetContactServer)(/*[in,out]*/ VARIANT *pServerName);
	STDMETHOD(FindNextUnit)();
	STDMETHOD(Decrement)(/*[in]*/ VARIANT writePassword, /*[in]*/ VARIANT address);
	STDMETHOD(Activate)(/*[in]*/ VARIANT writePassword, /*[in]*/ VARIANT activatepw1, /*[in]*/ VARIANT activatepw2, /*[in]*/ VARIANT address);
	STDMETHOD(SetContactServer)(/*[in]*/ VARIANT serverName);
	STDMETHOD(GetVersion)(/*[in,out]*/VARIANT *pMajor, /*[in,out]*/VARIANT *pMinor, /*[in,out]*/VARIANT *pRev, /*[in,out]*/VARIANT *pOsType);
	STDMETHOD(get_LastError)(/*[out, retval]*/ short *pVal);
	STDMETHOD(Initialize)();
	STDMETHOD(FormatPacket)();
	unsigned short int m_LastError;
	STDMETHOD(FindFirstUnit)(/*[in]*/ VARIANT developerId);
	//Active Server Pages Methods
	STDMETHOD(OnStartPage)(IUnknown* IUnk);
	STDMETHOD(OnEndPage)();

private:
	CSuperproContainer *m_SuperproContainer;
	CComPtr<IRequest> m_piRequest;					//Request Object
	CComPtr<IResponse> m_piResponse;				//Response Object
	CComPtr<ISessionObject> m_piSession;			//Session Object
	CComPtr<IServer> m_piServer;					//Server Object
	CComPtr<IApplicationObject> m_piApplication;	//Application Object
	BOOL m_bOnStartPageCalled;						//OnStartPage successful?
};

#endif //__SUPERPRO_H_
