// SuperproContainer.cpp: implementation of the CSuperproContainer class.
//
//////////////////////////////////////////////////////////////////////
/***************************************************************************************
*SuperProContainer.cpp
*--------------------------------------------------------------------------------------
*
* Description - Implementation file for SuperProContainer Class
*
* Created By  - Jogender Chauhan.
*
* Purpose     - Definition for all the member functions of this class
*				
* (C) Copyright 1991-2002 Rainbow Technologies, Inc. All rights reserved.
*-------------------------------------------------------------------------------------------
*
******************************************************************************************/

#include "stdafx.h"
#include "resource.h"
#include "SuperproContainer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSuperproContainer::CSuperproContainer()
{
}

CSuperproContainer::~CSuperproContainer()
{

}

/********************************************************************************
*
*	FUNCTION	: FormatPacket()
*
*	PURPOSE		: Format the packet for a client
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::FormatPacket()
{
	return (RNBOsproFormatPacket(m_ApiPacket, sizeof(m_ApiPacket)));
}

/********************************************************************************
*
*	FUNCTION	: FindFirstUnit()
*
*	PURPOSE		: For finding the key for a client
*
*	INPUT/OUTPUT: [IN] Dev ID of the key
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::FindFirstUnit(VARIANT developerId)
{
	SP_STATUS status;
	char *buffer;

	switch (developerId.vt)
	{
		case VT_BSTR:
			buffer = WideCharToMBCS(developerId.bstrVal);
			if (buffer == NULL)
				return SP_INVALID_PARAMETER;
			status = RNBOsproFindFirstUnit(m_ApiPacket, StrToHex(buffer));
			free (buffer);
			buffer = NULL;
			break;
		case VT_INT:
		case VT_I2:
		case VT_I4:
			status = RNBOsproFindFirstUnit(m_ApiPacket, developerId.intVal);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	return status;
}

/********************************************************************************
*
*	FUNCTION	: StrToHex()
*
*	PURPOSE		: For converting a array of characters to Hexadecimal value
*
*	INPUT/OUTPUT: [IN] pointer of a Array of char
*
*	RETURN		: Return the hex value
*
*********************************************************************************/

int CSuperproContainer::StrToHex(const char *value)
{
	struct CHexMap
	{
		char chr;
		int value;
	};

	const int HexMapL = 16;

	CHexMap HexMap[HexMapL] =
	{
		{'0', 0}, {'1', 1},
		{'2', 2}, {'3', 3},
		{'4', 4}, {'5', 5},
		{'6', 6}, {'7', 7},
		{'8', 8}, {'9', 9},
		{'A', 10}, {'B', 11},
		{'C', 12}, {'D', 13},
		{'E', 14}, {'F', 15}
	};

	char *mstr = _strupr(_strdup(value));
	char *s = mstr;
	int result = 0;
	if (*s == '0' && *(s + 1) == 'X') s += 2;
	bool firsttime = true;
	while (*s != '\0')
	{
		bool found = false;
		for (int i = 0; i < HexMapL; i++)
		{
			if (*s == HexMap[i].chr)
			{
				if (!firsttime) result <<= 4;
				result |= HexMap[i].value;
				found = true;
				break;
			}
		}

		if (!found) break;
		s++;
		firsttime = false;
	}
	free(mstr);
	return result;

}

/********************************************************************************
*
*	FUNCTION	: WideCharToMBCS()
*
*	PURPOSE		: Static Member function for converting a array of wide characters 
*				  to Multi Byte string
*
*	INPUT/OUTPUT: [IN] pointer of a Array of wide chars
*
*	RETURN		: Returns Array of MBCS chars
*
*********************************************************************************/

char * CSuperproContainer::WideCharToMBCS(wchar_t *wString)
{
	char	*tempStr=NULL;
	int		tempStrLen;
	
	if (wString == NULL)
	{
		tempStr=(char *) malloc (2);
		tempStr[0]='\0';
	}
	else
	{
		// first call gives us the number of bytes needed in the MBCS we are making.
		tempStrLen=WideCharToMultiByte (CP_ACP, 0, wString, -1, tempStr, 0, NULL, NULL);
		if (tempStrLen != 0)
		{
			// second call actually does the conversion.
			tempStr=(char *) malloc (tempStrLen);
			if (WideCharToMultiByte (CP_ACP, 0, wString, -1, tempStr, tempStrLen, NULL, NULL)==0)
			{
				// error occurred return an empty string
				free (tempStr);
				tempStr=NULL;
			}
		}
	}
	return tempStr;
}

/********************************************************************************
*
*	FUNCTION	: Initialize()
*
*	PURPOSE		: Initialize the SuperPro packet for a client
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Initialize()
{
	return (RNBOsproInitialize(m_ApiPacket));
}

/********************************************************************************
*
*	FUNCTION	: GetVersion()
*
*	PURPOSE		: Get the version of driver
*
*	INPUT/OUTPUT: returns Major Version,Minor version, Revision and OS type
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetVersion(VARIANT *pMajor, VARIANT *pMinor, VARIANT *pRev, VARIANT *pOsType)
{
	RB_BYTE maj, min, rev, ostype;
	SP_STATUS status;

	status = RNBOsproGetVersion(m_ApiPacket, &maj, &min, &rev, &ostype);
	if (status ==0)
	{
		pMajor->vt = VT_UI1;
		pMinor->vt = VT_UI1;
		pRev->vt = VT_UI1;
		pOsType->vt = VT_UI1;

		pMajor->bVal = maj;
		pMinor->bVal = min;
		pRev->bVal = rev;
		pOsType->bVal = ostype;
	}
	return status;
}

/********************************************************************************
*
*	FUNCTION	: SetContactServer()
*
*	PURPOSE		: Set the name of server.
*
*	INPUT/OUTPUT: [IN] server name
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::SetContactServer(VARIANT serverName)
{
	char *servername;
	SP_STATUS status;

	if (serverName.vt != VT_BSTR)
		return SP_INVALID_PARAMETER;

	servername = WideCharToMBCS(serverName.bstrVal);

	if (servername == NULL)
		status = SP_INVALID_PARAMETER;
	else
		status = RNBOsproSetContactServer(m_ApiPacket, servername);

	free(servername); servername = NULL;
	return status;
}

/********************************************************************************
*
*	FUNCTION	: Decrement()
*
*	PURPOSE		: To decrement the value of a cell.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] address of cell
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Decrement(VARIANT writePassword, VARIANT address)
{
	char *wp, *addr;
	SP_STATUS status;

	switch (writePassword.vt)
	{
		case VT_BSTR:
			if (address.vt == VT_BSTR)
			{
				wp = WideCharToMBCS(writePassword.bstrVal);
				if (wp == NULL)
					return SP_INVALID_PARAMETER;

				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
				{
					free(wp);
					wp = NULL;
					return SP_INVALID_PARAMETER;
				}

				status = RNBOsproDecrement(m_ApiPacket, StrToHex(wp), StrToHex(addr));
				free (wp); free (addr);
				wp = addr = NULL;
				break;
			} else if ((address.vt == VT_INT) || (address.vt == VT_I2) || (address.vt == VT_I4))
			{
				wp = WideCharToMBCS(writePassword.bstrVal);
				if (wp == NULL)
					return SP_INVALID_PARAMETER;

				status = RNBOsproDecrement(m_ApiPacket, StrToHex(wp), address.intVal );
				break;
			}else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ((address.vt == VT_INT) || (address.vt == VT_I2) || (address.vt == VT_I4))
			{
				status = RNBOsproDecrement(m_ApiPacket, writePassword.intVal, address.intVal);
				break;
			}else if (address.vt == VT_BSTR)
			{
				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
					return SP_INVALID_PARAMETER;

				status = RNBOsproDecrement(m_ApiPacket, writePassword.intVal, StrToHex(addr));
				free(addr);addr = NULL;
				break;
			}else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	return status;
}

/********************************************************************************
*
*	FUNCTION	: Activate()
*
*	PURPOSE		: To Activate a cell.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] address of cell
*				  [IN] activate Password1 of key, [IN] activate Password2
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Activate(VARIANT writePassword, VARIANT activatepw1, VARIANT activatepw2, VARIANT address)
{
	char *wp, *addr, *actpw1, *actpw2;
	SP_STATUS status;

	switch (writePassword.vt)
	{
		case VT_BSTR:
			if ( (activatepw1.vt == VT_BSTR) && (activatepw2.vt == VT_BSTR) && 
				(address.vt == VT_BSTR) )
			{
				wp = WideCharToMBCS(writePassword.bstrVal);
				if (wp == NULL)
					return SP_INVALID_PARAMETER;

				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
				{
					free(wp);
					wp = NULL;
					return SP_INVALID_PARAMETER;
				}
				
				actpw1 = WideCharToMBCS(activatepw1.bstrVal);
				if (actpw1 == NULL)
				{
					free(wp);free(addr);
					wp = addr = NULL;
					return SP_INVALID_PARAMETER;
				}

				actpw2 = WideCharToMBCS(activatepw2.bstrVal);
				if (actpw2 == NULL)
				{
					free(wp);free(addr);free(actpw1);
					wp = addr = actpw1 = NULL;
					return SP_INVALID_PARAMETER;
				}

				status = RNBOsproActivate(m_ApiPacket, StrToHex(wp), StrToHex(actpw1), 
									  StrToHex(actpw2), StrToHex(addr));

				free (wp); free (addr); free (actpw1); free (actpw2);
				wp = addr = actpw1 = actpw2 = NULL;
				break;
			} else {
				status =  SP_INVALID_PARAMETER;
				break;
			}
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ( ((activatepw1.vt == VT_INT) || (activatepw1.vt == VT_I2) || (activatepw1.vt == VT_I4)) && 
				((activatepw2.vt == VT_INT) || (activatepw2.vt == VT_I2) || (activatepw2.vt == VT_I4)) && 
				((address.vt == VT_INT) || (address.vt == VT_I2) || (address.vt == VT_I4)) )
			{

				status = RNBOsproActivate(m_ApiPacket, writePassword.intVal, activatepw1.intVal,
								   activatepw2.intVal, address.intVal);
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	return status;
}

/********************************************************************************
*
*	FUNCTION	: GetContactServer()
*
*	PURPOSE		: Get the name of server.
*
*	INPUT/OUTPUT: [OUT] server name
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetContactServer(VARIANT *pServerName)
{
	char *servername = new char[MAX_NAME_LEN + 1];
	SP_STATUS status;

	status = RNBOsproGetContactServer(m_ApiPacket, servername,  MAX_NAME_LEN);
	if (status ==0)
	{
		pServerName->vt = VT_BSTR;
		pServerName->bstrVal = SysAllocString(MBCSToWideChar(servername));
	}
	delete [] servername;
	servername = NULL;
	return status;
}

/********************************************************************************
*
*	FUNCTION	: GetFullStatus()
*
*	PURPOSE		: Get Last Error of API
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetFullStatus()
{
	return (RNBOsproGetFullStatus(m_ApiPacket));
}

/********************************************************************************
*
*	FUNCTION	: GetHardLimit()
*
*	PURPOSE		: Get the Hard Limit of a key
*
*	INPUT/OUTPUT: [OUT] HardLimit
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetHardLimit(VARIANT *pHardLimit)
{
	RB_WORD hardlimit;
	SP_STATUS status;

	status = RNBOsproGetHardLimit(m_ApiPacket, &hardlimit);

	if (status ==0)
	{
		pHardLimit->vt = VT_I2;
		pHardLimit->iVal = hardlimit;
	}
	return status;	
}

/********************************************************************************
*
*	FUNCTION	: GetSubLicense()
*
*	PURPOSE		: Get a sub license for a cell
*
*	INPUT/OUTPUT: [IN] Cell address
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetSubLicense(VARIANT address)
{
	SP_STATUS status;
	char *buffer;

	switch (address.vt)
	{
		case VT_BSTR:
			buffer = WideCharToMBCS(address.bstrVal);
			if (buffer == NULL)
				return SP_INVALID_PARAMETER;
			status = RNBOsproGetSubLicense(m_ApiPacket, StrToHex(buffer));
			free (buffer);
			buffer = NULL;
			break;
		case VT_INT:
		case VT_I2:
		case VT_I4:
			status = RNBOsproGetSubLicense(m_ApiPacket, address.intVal);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	return status;	
}

/********************************************************************************
*
*	FUNCTION	: Overwrite()
*
*	PURPOSE		: To Overwrite a cell by a given value.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] overwrite password 1
*				  [IN] overwrite password 2, [IN] Address of cell
*				  [IN] data, [IN] Access Code of the cell
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Overwrite(VARIANT writePassword, VARIANT owp1, VARIANT owp2, VARIANT address, VARIANT data, VARIANT accessCode)
{
	char *wp, *owpw1, *owpw2, *addr, *dat, *ac;
	SP_STATUS status;

	switch (writePassword.vt)		// Just check the first one and if it's a BSTR everything else is a BSTR
	{
		case VT_BSTR:
			if ( (owp1.vt == VT_BSTR) && (owp2.vt == VT_BSTR) && (address.vt == VT_BSTR) && 
				(data.vt == VT_BSTR) && (accessCode.vt == VT_BSTR) )
			{
				wp = WideCharToMBCS(writePassword.bstrVal);
				if (wp == NULL)
					return SP_INVALID_PARAMETER;

				owpw1 = WideCharToMBCS(owp1.bstrVal);
				if (owpw1 == NULL)
				{
					free(wp);
					wp = NULL;
					return SP_INVALID_PARAMETER;
				}

				owpw2 = WideCharToMBCS(owp2.bstrVal);
				if (owpw2 == NULL)
				{
					free(wp);free(owpw1);
					wp = owpw1 = NULL;
					return SP_INVALID_PARAMETER;
				}

				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
				{
					free(wp);free(owpw1);free(owpw2);
					wp = owpw1 = owpw2 = NULL;
					return SP_INVALID_PARAMETER;
				}

				dat = WideCharToMBCS(data.bstrVal);
				if (dat == NULL)
				{	
					free(wp);free(owpw1);free(owpw2);free(addr);
					wp = owpw1 = owpw2 = addr = NULL;
					return SP_INVALID_PARAMETER;
				}

				ac = WideCharToMBCS(accessCode.bstrVal);
				if (ac == NULL)
				{
					free(wp);free(owpw1);free(owpw2);free(addr);free(dat);
					wp = owpw1 = owpw2 = addr = dat = NULL;
					return SP_INVALID_PARAMETER;
				}

				status = RNBOsproOverwrite(m_ApiPacket, StrToHex(wp), StrToHex(owpw1),
										StrToHex(owpw2), StrToHex(addr), StrToHex(dat),
										StrToHex(ac));

				free (wp); free (owpw1); free (owpw2); free (addr); free (dat); free (ac);
				wp = addr = dat = ac = owpw1 = owpw2 = NULL;
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ( ((owp1.vt == VT_INT) || (owp1.vt == VT_I2) || (owp1.vt == VT_I4)) && 
				((owp2.vt == VT_INT) || (owp2.vt == VT_I2) || (owp2.vt == VT_I4)) && 
				((address.vt == VT_INT) || (address.vt == VT_I2) || (address.vt == VT_I4)) && 
				((data.vt == VT_INT) || (data.vt == VT_I2) || (data.vt == VT_I4)) && 
				((accessCode.vt == VT_INT) || (accessCode.vt == VT_I2) || (accessCode.vt == VT_I4)) )
			{
				status = RNBOsproOverwrite(m_ApiPacket, writePassword.intVal, owp1.intVal,
										owp2.intVal, address.intVal,data.intVal, 
										accessCode.intVal);
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	return status;
}

/********************************************************************************
*
*	FUNCTION	: Read()
*
*	PURPOSE		: To Read a value from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [OUT] Data
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Read(VARIANT address, VARIANT *pData)
{
	SP_STATUS status;
	char *buffer;
	RB_WORD data = 0;
	
	switch (address.vt)
	{
		case VT_BSTR:
			buffer = WideCharToMBCS(address.bstrVal);
			if (buffer == NULL)
				return SP_INVALID_PARAMETER;
			status = RNBOsproRead(m_ApiPacket, StrToHex(buffer), &data);
			free (buffer); 
			buffer = NULL;
			break;
		case VT_INT:
		case VT_I2:
		case VT_I4:
			status = RNBOsproRead(m_ApiPacket, address.intVal, &data);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	if (status ==0)
	{
		pData->vt = VT_I2;
		pData->iVal = data;
	}

	return status;
}

/********************************************************************************
*
*	FUNCTION	: ReleaseLicense()
*
*	PURPOSE		: To ReleaseLicense a license from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [IN/OUT] license to release/Actual license released
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::ReleaseLicense(VARIANT address, VARIANT *pNumOfLicense)
{
	SP_STATUS status;
	RB_WORD numOfLic = 0;
	char *buffer;
	char *temp;

	if ((pNumOfLicense->vt == VT_INT) || (pNumOfLicense->vt == VT_I2) || (pNumOfLicense->vt == VT_I4) )
		numOfLic = pNumOfLicense->intVal;
	else if (pNumOfLicense->vt == VT_BSTR)
	{
		temp = WideCharToMBCS(pNumOfLicense->bstrVal);
		if (temp == NULL)
			return SP_INVALID_PARAMETER;
		numOfLic = StrToHex(temp);
	} else
		return SP_INVALID_PARAMETER;

	switch (address.vt)
	{
		case VT_BSTR:
			buffer = WideCharToMBCS(address.bstrVal);
			if (buffer == NULL)
				return SP_INVALID_PARAMETER;
			status = RNBOsproReleaseLicense(m_ApiPacket, StrToHex(buffer), &numOfLic);
			free (buffer);
			buffer = NULL;
			break;
		case VT_INT:
		case VT_I2:
		case VT_I4:
			status = RNBOsproReleaseLicense(m_ApiPacket, address.intVal, &numOfLic);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	if (status ==0)
	{
		::VariantClear(pNumOfLicense);
		pNumOfLicense->vt = VT_I2;
		pNumOfLicense->iVal = numOfLic;
	}

	return status;	
}

/********************************************************************************
*
*	FUNCTION	: SetHeartBeat()
*
*	PURPOSE		: Set Heart beat for client..
*
*	INPUT/OUTPUT: [IN] heart beat value.
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::SetHeartBeat(VARIANT heartBeatValue)
{
	SP_STATUS status;

	switch (heartBeatValue.vt)
	{
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ((heartBeatValue.vt == VT_INT) || (heartBeatValue.vt == VT_I2) || (heartBeatValue.vt == VT_I4))
				status = RNBOsproSetHeartBeat(m_ApiPacket, heartBeatValue.intVal);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	return status;
}

/********************************************************************************
*
*	FUNCTION	: Write()
*
*	PURPOSE		: To write a cell by a given value.
*
*	INPUT/OUTPUT: [IN] Write Password of key, [IN] Address of cell
*				  [IN] data, [IN] Access Code of the cell
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Write(VARIANT writePassword, VARIANT address, VARIANT data, VARIANT accessCode)
{
	char *wp, *addr, *dat, *ac;
	SP_STATUS status;

	switch (writePassword.vt)
	{
		case VT_BSTR:
			if ( (address.vt == VT_BSTR) && (data.vt == VT_BSTR) && (accessCode.vt == VT_BSTR) )
			{
				wp = WideCharToMBCS(writePassword.bstrVal);
				if (wp == NULL)
					return SP_INVALID_PARAMETER;

				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
				{
					free(wp);
					wp = NULL;
					return SP_INVALID_PARAMETER;
				}

				dat = WideCharToMBCS(data.bstrVal);
				if (dat == NULL)
				{
					free(wp);free(addr);
					wp = addr = NULL;
					return SP_INVALID_PARAMETER;
				}

				ac = WideCharToMBCS(accessCode.bstrVal);
				if (ac == NULL)
				{
					free(wp);free(addr);free(dat);
					wp = addr = dat = NULL;
					return SP_INVALID_PARAMETER;
				}

				status = RNBOsproWrite(m_ApiPacket, StrToHex(wp), StrToHex(addr), StrToHex(dat),
								   StrToHex(ac));
				free (wp); free (addr); free (dat); free (ac);
				wp = addr = dat = ac = NULL;
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ( ((address.vt == VT_INT) || (address.vt == VT_I2) || (address.vt == VT_I4)) && 
				((data.vt == VT_INT) || (data.vt == VT_I2) || (data.vt == VT_I4)) && 
				((accessCode.vt == VT_INT) || (accessCode.vt == VT_I2) || (accessCode.vt == VT_I4)) )
			{
				status = RNBOsproWrite(m_ApiPacket, writePassword.intVal, address.intVal,
								   data.intVal, accessCode.intVal);
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	
	return status;
}

/********************************************************************************
*
*	FUNCTION	: FindNextUnit()
*
*	PURPOSE		: For finding the another key of same devID for a client
*
*	INPUT/OUTPUT: None
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::FindNextUnit()
{
	return (RNBOsproFindNextUnit(m_ApiPacket));
}

/********************************************************************************
*
*	FUNCTION	: MBCSToWideChar()
*
*	PURPOSE		: Static Member function for converting a array of MBCS characters 
*				  to wide string
*
*	INPUT/OUTPUT: [IN] pointer of a Array of MBCS chars
*
*	RETURN		: Returns Array of Wide chars
*
*********************************************************************************/

wchar_t * CSuperproContainer::MBCSToWideChar(const char *mbcsString)
{
	wchar_t	*tempStr=NULL;
	int		tempStrLen;
	
	if (mbcsString == NULL)
	{
		tempStr=(wchar_t *) malloc (2);
		tempStr[0]='\0';
	}
	else
	{
		// first call gives us the number of bytes needed in the WCHAR_T we are making.
		tempStrLen = MultiByteToWideChar (CP_ACP, 0, mbcsString, -1, tempStr, 0);
		if (tempStrLen != 0)
		{
			// second call actually does the conversion.
			tempStr = (wchar_t *) malloc (tempStrLen * 2);
			if (MultiByteToWideChar (CP_ACP, 0, mbcsString, -1, tempStr, tempStrLen) == 0)
			{
				// error occurred return an empty string
				free (tempStr);
				tempStr=NULL;
			}
		}
	}
	return tempStr;
}

/********************************************************************************
*
*	FUNCTION	: ExtendedRead()
*
*	PURPOSE		: To Read a value from a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [OUT] Data, [OUT] Access code
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::ExtendedRead(VARIANT address, VARIANT *pData, VARIANT *pAccessCode)
{
	SP_STATUS status;
	char *buffer;
	RB_WORD data = 0;
	RB_BYTE accessCode = 0;
	
	switch (address.vt)
	{
		case VT_BSTR:
			buffer = WideCharToMBCS(address.bstrVal);
			if (buffer == NULL)
				return SP_INVALID_PARAMETER;
			status = RNBOsproExtendedRead(m_ApiPacket, StrToHex(buffer), &data, &accessCode);
			free (buffer); 
			buffer = NULL;
			break;
		case VT_INT:
		case VT_I4:
		case VT_I2:
			status = RNBOsproExtendedRead(m_ApiPacket, address.intVal, &data, &accessCode);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	if (status ==0)
	{
		pData->vt = VT_I2;
		pData->iVal = data;
		pAccessCode->vt = VT_UI1;
		pAccessCode->bVal = accessCode;
	} 

	return status;
}

/********************************************************************************
*
*	FUNCTION	: SetProtocol()
*
*	PURPOSE		: Set the protocol for client/server communication.
*
*	INPUT/OUTPUT: [IN] Protocol
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::SetProtocol(VARIANT protocol)
{
	SP_STATUS status;
	switch (protocol.vt)
	{
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ((protocol.vt == VT_INT) || (protocol.vt == VT_I2) || (protocol.vt == VT_I4))
				status = RNBOsproSetProtocol(m_ApiPacket,protocol.intVal);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	return status;
}

/********************************************************************************
*
*	FUNCTION	: Query()
*
*	PURPOSE		: For query on a given cell address.
*
*	INPUT/OUTPUT: [IN] Address of cell, [IN] query Data, [OUT] Response
*				  [OUT] response32 (last 4 byte of response), [OUT] length of response
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::Query(VARIANT address, VARIANT queryData, VARIANT *pResponse, VARIANT *pResponse32, VARIANT length)
{
	char *addr, *plen;
	char *queryBuffer; 
	char queryString[57] = "\0";
	char responseString[57] = "\0";
	char displayString[57] = "\0";
	DWORD resp32 = 0;
	SP_STATUS status;

	switch (address.vt)
	{
		case VT_BSTR:
			if (length.vt == VT_BSTR)
			{
				addr = WideCharToMBCS(address.bstrVal);
				if (addr == NULL)
					return SP_INVALID_PARAMETER;

				plen = WideCharToMBCS(length.bstrVal);
				if (plen == NULL)
				{
					free(addr);
					addr = NULL;
					return SP_INVALID_PARAMETER;
				}
				
				if (queryData.vt == VT_BSTR)
				{
					queryBuffer = WideCharToMBCS(queryData.bstrVal);
					if (queryBuffer == NULL)
					{
						free(addr);free(plen);
						addr = plen = NULL;
						return SP_INVALID_PARAMETER;
					}
					ConvertQueryStr(queryBuffer, queryString);
				} else {
					status = SP_INVALID_PARAMETER;
					break;
				}

				status = RNBOsproQuery(m_ApiPacket, StrToHex(addr), queryString, responseString, 
										&resp32, StrToHex(plen));
				free (addr); free (plen);
				addr = plen = NULL;
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;	
			}
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ( (length.vt == VT_INT) || (length.vt == VT_I2) || (length.vt == VT_I4 ))
			{
				if (queryData.vt == VT_BSTR)
				{
					queryBuffer = WideCharToMBCS(queryData.bstrVal);
					ConvertQueryStr(queryBuffer, queryString);
				} else {
					status = SP_INVALID_PARAMETER;
					break;
				}

				status = RNBOsproQuery(m_ApiPacket, address.intVal, queryString, responseString, 
										&resp32, length.intVal);
				break;
			} else {
				status = SP_INVALID_PARAMETER;
				break;
			}

		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	if (status ==0)
	{
		ConvertResponseStr(responseString, displayString, strlen(responseString));
		pResponse->vt = VT_BSTR;
		pResponse->bstrVal = SysAllocString(MBCSToWideChar(displayString));
		pResponse32->vt = VT_I4;
		pResponse32->lVal = 0;
	}
	return status;
}

void CSuperproContainer::ConvertQueryStr(char *queryStr, char *realQueryStr)
{
	unsigned int i = 0, j = 0;
	unsigned char digit, num;

	memset(realQueryStr, 0, sizeof(realQueryStr));
	while (i < strlen (queryStr)) 
	{
		digit = HexCharToDigit (queryStr [i]);
		num = digit;
		/* the first char is the high order hex byte */
		num = num << 4;
		i++;
		digit = HexCharToDigit (queryStr [i]);
		/* add the low order byte */
		num += digit;
		i++;
		realQueryStr [j] = num;
		j++;
	} /* while */

	realQueryStr [j] = '\0';
}

void CSuperproContainer::ConvertResponseStr(char *responseStr, char *disResponseStr, int len)
{
	int i = 0, j = 0;
	unsigned char digit;

	memset(disResponseStr, 0, sizeof(disResponseStr));
	while (i < len) {
		// get the hi order hex digit 
		digit = ( (unsigned char) responseStr[i]) >> 4;
	     disResponseStr [j] = MakeHexChar (digit);
		 j++;
	     // get the lo order hex digit 
		 digit = responseStr[i] & 0x0F;
	     disResponseStr [j] = MakeHexChar (digit);
		 j++;
	     i++;
	} 

	  /* null terminate the string for display */
	  disResponseStr [j] = '\0';
}

unsigned char CSuperproContainer::HexCharToDigit(char hexChar)
{
	unsigned char digit;

	if ( ('0' <= hexChar) && (hexChar <= '9') )
		digit = hexChar - '0';
	else
		digit = hexChar - 'A' + 10;
	return digit;
}

unsigned char CSuperproContainer::MakeHexChar(unsigned char num)
{
	unsigned char hexChar;
	if (num < 10)
		hexChar = '0' + num;
	else
		if (num <= 15)
			hexChar = 'A' + num - 10;
		else
			hexChar = 'F';
		return hexChar;

}

/********************************************************************************
*
*	FUNCTION	: GetKeyInfo()
*
*	PURPOSE		: For getting the information of key.
*
*	INPUT/OUTPUT: [IN] Dev ID, [IN] Key Index, [OUT] structure of key info
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::GetKeyInfo(VARIANT developerId, VARIANT keyIndex, NSPRO_MONITOR_INFO *pBufferInfo)
{
	SP_STATUS status;
	char *devid;
	char *kIndex;

	switch (developerId.vt)
	{
		case VT_BSTR: 
			devid = WideCharToMBCS(developerId.bstrVal);
			if (devid == NULL)
				status = SP_INVALID_PARAMETER;

			if (keyIndex.vt == VT_BSTR)
			{
				kIndex = WideCharToMBCS(keyIndex.bstrVal);
				if (kIndex == NULL)
					status = SP_INVALID_PARAMETER;

				status = RNBOsproGetKeyInfo(m_ApiPacket, StrToHex(devid), StrToHex(kIndex), pBufferInfo);
			}else if ((keyIndex.vt == VT_INT)||(keyIndex.vt == VT_I2)||(keyIndex.vt == VT_I4))
				status = RNBOsproGetKeyInfo(m_ApiPacket, StrToHex(devid), keyIndex.iVal, pBufferInfo);

			break;
		case VT_INT:
		case VT_I2:
		case VT_I4:
			if ( (keyIndex.vt == VT_INT) || (keyIndex.vt == VT_I2) || (keyIndex.vt == VT_I4) ) //if all inputs are int
				status = RNBOsproGetKeyInfo(m_ApiPacket, developerId.intVal, keyIndex.iVal, pBufferInfo);
			break;
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}
	
	return status;
}

/********************************************************************************
*
*	FUNCTION	: EnumServer()
*
*	PURPOSE		: For getting the list of running servers in subnet.
*
*	INPUT/OUTPUT: [IN] flag , [IN] Dev ID, [OUT] structure of server list
*
*	RETURN		: Return the status of SuperPro API
*
*********************************************************************************/

SP_STATUS CSuperproContainer::EnumServer(VARIANT enumflag, VARIANT developerId, NSPRO_SERVER_INFO *pServersInfoRecord, VARIANT *pNumOfServers)
{
	SP_STATUS status;
	RB_WORD numOfServers = 0;
	char *enumFlag;
	char *devId;
	char *temp;

	//do manipulation for number of servers
	if(pNumOfServers->vt == VT_BSTR)
	{
		temp =WideCharToMBCS(pNumOfServers->bstrVal);
		if(temp == NULL)
			return SP_INVALID_PARAMETER;

		numOfServers = sscanf("%d",temp);
	}else if ((pNumOfServers->vt = VT_INT)||(pNumOfServers->vt = VT_I2)||(pNumOfServers->vt = VT_I4))
		numOfServers = pNumOfServers->intVal;

	switch (enumflag.vt)
	{
		case VT_BSTR:
			enumFlag = WideCharToMBCS(enumflag.bstrVal);
			if (enumFlag == NULL)
				return SP_INVALID_PARAMETER;

			if (developerId.vt == VT_BSTR)
			{
				devId = WideCharToMBCS(developerId.bstrVal);
				if (devId == NULL)
				{
					free(devId);devId = NULL;
					return SP_INVALID_PARAMETER;
				}
				status = RNBOsproEnumServer(StrToHex(enumFlag), StrToHex(devId), pServersInfoRecord, &numOfServers);
			} else if ((developerId.vt == VT_INT)||(developerId.vt == VT_I2)||(developerId.vt == VT_I4))
					status = RNBOsproEnumServer(StrToHex(enumFlag), developerId.intVal, pServersInfoRecord, &numOfServers);

			free (enumFlag); free (devId);
			enumFlag = NULL; devId = NULL;
			break;
		case VT_INT:
		case VT_I4:
		case VT_I2:
			if ( (developerId.vt == VT_INT) || (developerId.vt == VT_I2) || (developerId.vt == VT_I4) )
			{
				status = RNBOsproEnumServer(enumflag.iVal, developerId.intVal, pServersInfoRecord, &numOfServers);
				break;
			}else if (developerId.vt == VT_BSTR)
			{
				devId = WideCharToMBCS(developerId.bstrVal);
				if (devId == NULL)
				{
					free(devId); devId = NULL;
					return SP_INVALID_PARAMETER;
				}
				status = RNBOsproEnumServer(enumflag.intVal, StrToHex(devId), pServersInfoRecord, &numOfServers);
				break;
			}
		default:
			status = SP_INVALID_PARAMETER;
			break;
	}

	if (status ==0)
	{
		pNumOfServers->vt = VT_INT;
		pNumOfServers->intVal = numOfServers;
	}
	return status;
}
